@@ -509,7 +509,7 @@ class WP_Rewrite {
 		$date_endian          = '';
 
 		foreach ( $endians as $endian ) {
-			if ( false !== strpos( $this->permalink_structure, $endian ) ) {
+			if ( str_contains( $this->permalink_structure, $endian ) ) {
 				$date_endian = $endian;
 				break;
 			}
@@ -531,7 +531,7 @@ class WP_Rewrite {
 				$front = $front . 'date/';
 				break;
 			}
-			$tok_index++;
+			++$tok_index;
 		}
 
 		$this->date_structure = $front . $date_endian;
@@ -1060,13 +1060,18 @@ class WP_Rewrite {
 				 * 2) post ID, 3) page name, 4) timestamp (year, month, day, hour, second and
 				 * minute all present). Set these flags now as we need them for the endpoints.
 				 */
-				if ( strpos( $struct, '%postname%' ) !== false
-						|| strpos( $struct, '%post_id%' ) !== false
-						|| strpos( $struct, '%pagename%' ) !== false
-						|| ( strpos( $struct, '%year%' ) !== false && strpos( $struct, '%monthnum%' ) !== false && strpos( $struct, '%day%' ) !== false && strpos( $struct, '%hour%' ) !== false && strpos( $struct, '%minute%' ) !== false && strpos( $struct, '%second%' ) !== false )
-						) {
+				if ( str_contains( $struct, '%postname%' )
+					|| str_contains( $struct, '%post_id%' )
+					|| str_contains( $struct, '%pagename%' )
+					|| ( str_contains( $struct, '%year%' )
+						&& str_contains( $struct, '%monthnum%' )
+						&& str_contains( $struct, '%day%' )
+						&& str_contains( $struct, '%hour%' )
+						&& str_contains( $struct, '%minute%' )
+						&& str_contains( $struct, '%second%' ) )
+				) {
 					$post = true;
-					if ( strpos( $struct, '%pagename%' ) !== false ) {
+					if ( str_contains( $struct, '%pagename%' ) ) {
 						$page = true;
 					}
 				}
@@ -1074,7 +1079,7 @@ class WP_Rewrite {
 				if ( ! $post ) {
 					// For custom post types, we need to add on endpoints as well.
 					foreach ( get_post_types( array( '_builtin' => false ) ) as $ptype ) {
-						if ( strpos( $struct, "%$ptype%" ) !== false ) {
+						if ( str_contains( $struct, "%$ptype%" ) ) {
 							$post = true;
 
 							// This is for page style attachment URLs.
@@ -1282,6 +1287,9 @@ class WP_Rewrite {
 		// favicon.ico -- only if installed at the root.
 		$favicon_rewrite = ( empty( $home_path['path'] ) || '/' === $home_path['path'] ) ? array( 'favicon\.ico$' => $this->index . '?favicon=1' ) : array();
 
+		// sitemap.xml -- only if installed at the root.
+		$sitemap_rewrite = ( empty( $home_path['path'] ) || '/' === $home_path['path'] ) ? array( 'sitemap\.xml' => $this->index . '??sitemap=index' ) : array();
+
 		// Old feed and service files.
 		$deprecated_files = array(
 			'.*wp-(atom|rdf|rss|rss2|feed|commentsrss2)\.php$' => $this->index . '?feed=old',
@@ -1399,7 +1407,7 @@ class WP_Rewrite {
 		// Extra permastructs.
 		foreach ( $this->extra_permastructs as $permastructname => $struct ) {
 			if ( is_array( $struct ) ) {
-				if ( count( $struct ) == 2 ) {
+				if ( count( $struct ) === 2 ) {
 					$rules = $this->generate_rewrite_rules( $struct[0], $struct[1] );
 				} else {
 					$rules = $this->generate_rewrite_rules( $struct['struct'], $struct['ep_mask'], $struct['paged'], $struct['feed'], $struct['forcomments'], $struct['walk_dirs'], $struct['endpoints'] );
@@ -1444,9 +1452,9 @@ class WP_Rewrite {
 
 		// Put them together.
 		if ( $this->use_verbose_page_rules ) {
-			$this->rules = array_merge( $this->extra_rules_top, $robots_rewrite, $favicon_rewrite, $deprecated_files, $registration_pages, $root_rewrite, $comments_rewrite, $search_rewrite, $author_rewrite, $date_rewrite, $page_rewrite, $post_rewrite, $this->extra_rules );
+			$this->rules = array_merge( $this->extra_rules_top, $robots_rewrite, $favicon_rewrite, $sitemap_rewrite, $deprecated_files, $registration_pages, $root_rewrite, $comments_rewrite, $search_rewrite, $author_rewrite, $date_rewrite, $page_rewrite, $post_rewrite, $this->extra_rules );
 		} else {
-			$this->rules = array_merge( $this->extra_rules_top, $robots_rewrite, $favicon_rewrite, $deprecated_files, $registration_pages, $root_rewrite, $comments_rewrite, $search_rewrite, $author_rewrite, $date_rewrite, $post_rewrite, $page_rewrite, $this->extra_rules );
+			$this->rules = array_merge( $this->extra_rules_top, $robots_rewrite, $favicon_rewrite, $sitemap_rewrite, $deprecated_files, $registration_pages, $root_rewrite, $comments_rewrite, $search_rewrite, $author_rewrite, $date_rewrite, $post_rewrite, $page_rewrite, $this->extra_rules );
 		}
 
 		/**
@@ -1485,18 +1493,36 @@ class WP_Rewrite {
 	public function wp_rewrite_rules() {
 		$this->rules = get_option( 'rewrite_rules' );
 		if ( empty( $this->rules ) ) {
-			$this->matches = 'matches';
-			$this->rewrite_rules();
-			if ( ! did_action( 'wp_loaded' ) ) {
-				add_action( 'wp_loaded', array( $this, 'flush_rules' ) );
-				return $this->rules;
-			}
-			update_option( 'rewrite_rules', $this->rules );
+			$this->refresh_rewrite_rules();
 		}
 
 		return $this->rules;
 	}
 
+	/**
+	 * Refreshes the rewrite rules, saving the fresh value to the database.
+	 *
+	 * If the {@see 'wp_loaded'} action has not occurred yet, will postpone saving to the database.
+	 *
+	 * @since 6.4.0
+	 */
+	private function refresh_rewrite_rules() {
+		$this->rules   = '';
+		$this->matches = 'matches';
+
+		$this->rewrite_rules();
+
+		if ( ! did_action( 'wp_loaded' ) ) {
+			/*
+			 * It is not safe to save the results right now, as the rules may be partial.
+			 * Need to give all rules the chance to register.
+			 */
+			add_action( 'wp_loaded', array( $this, 'flush_rules' ) );
+		} else {
+			update_option( 'rewrite_rules', $this->rules );
+		}
+	}
+
 	/**
 	 * Retrieves mod_rewrite-formatted rewrite rules to write to .htaccess.
 	 *
@@ -1555,7 +1581,7 @@ class WP_Rewrite {
 				// Apache 1.3 does not support the reluctant (non-greedy) modifier.
 				$match = str_replace( '.+?', '.+', $match );
 
-				if ( strpos( $query, $this->index ) !== false ) {
+				if ( str_contains( $query, $this->index ) ) {
 					$rules .= 'RewriteRule ^' . $match . ' ' . $home_root . $query . " [QSA,L]\n";
 				} else {
 					$rules .= 'RewriteRule ^' . $match . ' ' . $site_root . $query . " [QSA,L]\n";
@@ -1660,10 +1686,10 @@ class WP_Rewrite {
 			$external = false;
 			$query    = add_query_arg( $query, 'index.php' );
 		} else {
-			$index = false === strpos( $query, '?' ) ? strlen( $query ) : strpos( $query, '?' );
+			$index = ! str_contains( $query, '?' ) ? strlen( $query ) : strpos( $query, '?' );
 			$front = substr( $query, 0, $index );
 
-			$external = $front != $this->index;
+			$external = $front !== $this->index;
 		}
 
 		// "external" = it doesn't correspond to index.php.
@@ -1794,7 +1820,8 @@ class WP_Rewrite {
 		if ( ! is_array( $args ) ) {
 			$args = array( 'with_front' => $args );
 		}
-		if ( func_num_args() == 4 ) {
+
+		if ( func_num_args() === 4 ) {
 			$args['ep_mask'] = func_get_arg( 3 );
 		}
 
@@ -1807,14 +1834,16 @@ class WP_Rewrite {
 			'walk_dirs'   => true,
 			'endpoints'   => true,
 		);
-		$args     = array_intersect_key( $args, $defaults );
-		$args     = wp_parse_args( $args, $defaults );
+
+		$args = array_intersect_key( $args, $defaults );
+		$args = wp_parse_args( $args, $defaults );
 
 		if ( $args['with_front'] ) {
 			$struct = $this->front . $struct;
 		} else {
 			$struct = $this->root . $struct;
 		}
+
 		$args['struct'] = $struct;
 
 		$this->extra_permastructs[ $name ] = $args;
@@ -1856,8 +1885,7 @@ class WP_Rewrite {
 			unset( $do_hard_later );
 		}
 
-		update_option( 'rewrite_rules', '' );
-		$this->wp_rewrite_rules();
+		$this->refresh_rewrite_rules();
 
 		/**
 		 * Filters whether a "hard" rewrite rule flush should be performed when requested.
@@ -1907,7 +1935,7 @@ class WP_Rewrite {
 		unset( $this->feed_structure );
 		unset( $this->comment_feed_structure );
 
-		$this->use_trailing_slashes = ( '/' === substr( $this->permalink_structure, -1, 1 ) );
+		$this->use_trailing_slashes = str_ends_with( $this->permalink_structure, '/' );
 
 		// Enable generic rules for pages if permalink structure doesn't begin with a wildcard.
 		if ( preg_match( '/^[^%]*%(?:postname|category|tag|author)%/', $this->permalink_structure ) ) {
@@ -1932,7 +1960,7 @@ class WP_Rewrite {
 	 * @param string $permalink_structure Permalink structure.
 	 */
 	public function set_permalink_structure( $permalink_structure ) {
-		if ( $permalink_structure != $this->permalink_structure ) {
+		if ( $this->permalink_structure !== $permalink_structure ) {
 			$old_permalink_structure = $this->permalink_structure;
 			update_option( 'permalink_structure', $permalink_structure );
 
