@@ -559,7 +559,7 @@ window.autosave = function() {
 						});
 					} else {
 
-						// Make sure the Text editor is selected.
+						// Make sure the Code editor is selected.
 						$( '#content-html' ).trigger( 'click' );
 						$( '#content' ).trigger( 'focus' );
 
@@ -582,7 +582,7 @@ window.autosave = function() {
 			 * Don't run if the post type supports neither 'editor' (textarea#content) nor 'excerpt'.
 			 */
 			if ( checkStorage() && blog_id && ( $('#content').length || $('#excerpt').length ) ) {
-				$document.ready( run );
+				$( run );
 			}
 
 			return {
@@ -798,7 +798,9 @@ window.autosave = function() {
 			 *
 			 * @return {void}
 			 */
-			$document.on( 'heartbeat-send.autosave', function( event, data ) {
+			$( function() {
+				_schedule();
+			}).on( 'heartbeat-send.autosave', function( event, data ) {
 				var autosaveData = save();
 
 				if ( autosaveData ) {
@@ -848,8 +850,6 @@ window.autosave = function() {
 			}).on( 'heartbeat-connection-restored.autosave', function() {
 				$('#lost-connection-notice').hide();
 				enableButtons();
-			}).ready( function() {
-				_schedule();
 			});
 
 			return {
@@ -873,7 +873,10 @@ window.autosave = function() {
 		 *
 		 * @return {void}
 		 */
-		$document.on( 'tinymce-editor-init.autosave', function( event, editor ) {
+		$( function() {
+			// Set the initial compare string in case TinyMCE is not used or not loaded first.
+			setInitialCompare();
+		}).on( 'tinymce-editor-init.autosave', function( event, editor ) {
 			// Reset the initialCompare data after the TinyMCE instances have been initialized.
 			if ( 'content' === editor.id || 'excerpt' === editor.id ) {
 				window.setTimeout( function() {
@@ -881,9 +884,6 @@ window.autosave = function() {
 					setInitialCompare();
 				}, 1000 );
 			}
-		}).ready( function() {
-			// Set the initial compare string in case TinyMCE is not used or not loaded first.
-			setInitialCompare();
 		});
 
 		return {
