@@ -16,21 +16,21 @@
  *
  * @global string $wp_version
  */
-$wp_version = '6.6.1';
+$wp_version = '6.8.3';
 
 /**
  * Holds the WordPress DB revision, increments when changes are made to the WordPress DB schema.
  *
  * @global int $wp_db_version
  */
-$wp_db_version = 57155;
+$wp_db_version = 60421;
 
 /**
  * Holds the TinyMCE version.
  *
  * @global string $tinymce_version
  */
-$tinymce_version = '49110-20201110';
+$tinymce_version = '49110-20250317';
 
 /**
  * Holds the required PHP version.
@@ -39,6 +39,16 @@ $tinymce_version = '49110-20201110';
  */
 $required_php_version = '7.2.24';
 
+/**
+ * Holds the names of required PHP extensions.
+ *
+ * @global string[] $required_php_extensions
+ */
+$required_php_extensions = array(
+	'json',
+	'hash',
+);
+
 /**
  * Holds the required MySQL version.
  *
